@@ -1,40 +1,47 @@
 """ Integration tests for DataStructureElement API
 """
-from bson import ObjectId
-from django.contrib.auth.models import AnonymousUser
 from unittest.mock import Mock
 
+from bson import ObjectId
+from django.contrib.auth.models import AnonymousUser
 from django.http import HttpRequest
 
 from core_main_app.access_control.exceptions import AccessControlError
 from core_main_app.commons.exceptions import DoesNotExist
-from core_parser_app.components.data_structure_element import (
-    api as data_structure_element_api,
-)
-
 from core_main_app.utils.integration_tests.integration_base_test_case import (
     MongoIntegrationBaseTestCase,
 )
-from core_main_app.utils.tests_tools.MockUser import create_mock_user
+from core_parser_app.access_control import _check_data_structure_elements_access
+from core_parser_app.components.data_structure_element import (
+    api as data_structure_element_api,
+)
 from core_parser_app.components.data_structure_element.models import (
     DataStructureElement,
 )
 from tests.components.data_structure_element.fixtures.fixtures import (
     DataStructureElementFixtures,
 )
+from tests.fixtures_utils import MockAnonDataStructure
 
 
 class TestUpsert(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
         self.mock_data_structure_element = DataStructureElement(
-            id=ObjectId(), user="1", tag="mock_tag", value="mock_value"
+            id=ObjectId(),
+            user=str(self.users["owner"].id),
+            tag="mock_tag",
+            value="mock_value",
+            data_structure=self.fixtures.data_structure,
         )
 
         self.mock_request = Mock(spec=HttpRequest)
@@ -102,16 +109,16 @@ class TestUpsert(MongoIntegrationBaseTestCase):
 
 class TestGetAllByChildId(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
-        self.fixtures = DataStructureElementFixtures()
-        self.fixtures.insert_data()
-
         self.mock_request = Mock(spec=HttpRequest)
 
     def test_anonymous_cannot_perform_operation(self):
@@ -169,16 +176,16 @@ class TestGetAllByChildId(MongoIntegrationBaseTestCase):
 
 class TestGetById(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
-        self.fixtures = DataStructureElementFixtures()
-        self.fixtures.insert_data()
-
         self.mock_request = Mock(spec=HttpRequest)
 
     def test_anonymous_cannot_perform_operation(self):
@@ -224,16 +231,16 @@ class TestGetById(MongoIntegrationBaseTestCase):
 
 class TestGetByXPath(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
-        self.fixtures = DataStructureElementFixtures()
-        self.fixtures.insert_data()
-
         self.mock_request = Mock(spec=HttpRequest)
 
     def test_anonymous_cannot_perform_operation(self):
@@ -293,16 +300,16 @@ class TestGetByXPath(MongoIntegrationBaseTestCase):
 
 class TestRemoveChild(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
-        self.fixtures = DataStructureElementFixtures()
-        self.fixtures.insert_data()
-
         self.mock_request = Mock(spec=HttpRequest)
 
     def test_anonymous_cannot_perform_operation(self):
@@ -364,20 +371,22 @@ class TestRemoveChild(MongoIntegrationBaseTestCase):
 
 class TestAddChild(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
         self.mock_data_structure_element = DataStructureElement(
-            user="1", tag="mock_tag", value="mock_value"
+            user=str(self.users["owner"].id),
+            tag="mock_tag",
+            value="mock_value",
+            data_structure=self.fixtures.data_structure,
         ).save()
-
-        self.fixtures = DataStructureElementFixtures()
-        self.fixtures.insert_data()
-
         self.mock_request = Mock(spec=HttpRequest)
 
     def test_anonymous_cannot_perform_operation(self):
@@ -439,16 +448,16 @@ class TestAddChild(MongoIntegrationBaseTestCase):
 
 class TestGetRootElement(MongoIntegrationBaseTestCase):
     def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
         self.users = {
             "anon": AnonymousUser(),
-            "user": create_mock_user(user_id=0),
-            "owner": create_mock_user(user_id=1),
-            "superuser": create_mock_user(user_id=2, is_superuser=True),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
         }
 
-        self.fixtures = DataStructureElementFixtures()
-        self.fixtures.insert_data()
-
         self.mock_request = Mock(spec=HttpRequest)
 
     def test_anonymous_cannot_perform_operation(self):
@@ -492,3 +501,75 @@ class TestGetRootElement(MongoIntegrationBaseTestCase):
             result,
             self.fixtures.data_structure_element_collection["root"],
         )
+
+
+class TestDataStructureElementPermissionWithOwners(MongoIntegrationBaseTestCase):
+    """Test access to data structure elements when owner is set"""
+
+    def setUp(self):
+        self.fixtures = DataStructureElementFixtures()
+        self.fixtures.insert_data(user=self.fixtures.default_owner_with_perm)
+
+        self.users = {
+            "anon": AnonymousUser(),
+            "user": self.fixtures.other_user_with_perm,
+            "owner": self.fixtures.default_owner_with_perm,
+            "superuser": self.fixtures.superuser,
+        }
+
+        self.mock_request = Mock(spec=HttpRequest)
+
+    def test_anon_user_can_not_access_dse_if_permission_and_owner_is_set(self):
+        anon_user_with_perm = AnonymousUser()
+        self.mock_request.user = anon_user_with_perm
+
+        with self.assertRaises(AccessControlError):
+            _check_data_structure_elements_access(
+                [self.fixtures.data_structure_element_collection["root"]],
+                anon_user_with_perm,
+            )
+
+    def test_owner_can_access_dse_if_permission_and_owner_is_set(self):
+        _check_data_structure_elements_access(
+            [self.fixtures.data_structure_element_collection["root"]],
+            self.users["owner"],
+        )
+
+    def test_user_can_not_access_dse_if_permission_and_not_owner(self):
+        with self.assertRaises(AccessControlError):
+            _check_data_structure_elements_access(
+                [self.fixtures.data_structure_element_collection["root"]],
+                self.users["user"],
+            )
+
+
+class TestDataStructureElementPermissionWithoutOwners(MongoIntegrationBaseTestCase):
+    """Test access to data structure elements when owner is not set"""
+
+    def setUp(self):
+        self.fixtures = DataStructureElementFixtures(add_anon_perm=True)
+        self.fixtures.insert_data(
+            user=AnonymousUser(), data_structure_class=MockAnonDataStructure
+        )
+
+    def test_anon_user_can_access_dse_if_permission_and_no_owner(self):
+        _check_data_structure_elements_access(
+            [self.fixtures.data_structure_element_collection["root"]], AnonymousUser()
+        )
+
+
+class TestDataStructureElementPermissionWithoutOwnersWithoutPerm(
+    MongoIntegrationBaseTestCase
+):
+    """Test access to data structure elements when owner is not set"""
+
+    def setUp(self):
+        self.fixtures = DataStructureElementFixtures(add_anon_perm=False)
+        self.fixtures.insert_data(user=AnonymousUser())
+
+    def test_anon_user_can_not_access_dse_if_no_permission_and_no_owner(self):
+        with self.assertRaises(AccessControlError):
+            _check_data_structure_elements_access(
+                [self.fixtures.data_structure_element_collection["root"]],
+                AnonymousUser(),
+            )
